# (c) I. Marti-Vidal (Univ. Valencia) 2020.

# Script to solve for the instrumental polarization of (synthetic)
# EHT observations of a resolved calibrator, using the "source slicing".
# strategy, as originally implemented in LPCAL (Leppanen 1995).
# This script complements the contents of the paper:

# I. Marti-Vidal, A. Mus, P. de Vicente & J. Gonzalez (2020), ... TBC





import numpy as np
import pylab as pl
import os, sys
import glob
import pickle as pk
from task_polsolve import polsolve as polsolve


# Name of measurment set:
MSNAME = 'POLSIM_CASE-33.ms'


# Solve for Dterms and POINT source polarization:
polsolve(vis = MSNAME,
         mounts=['AZ', 'AZ', 'AZ', 'AZ', 'AZ', 'AZ', 'AZ', 'AZ'],
## The source is just a point source at the phase center:
         CLEAN_models=[1.0],
## Initial guess of the polarization state:
         frac_pol=[0.], EVPA=[0.], 
## Solve for the polarization of the source (together with the Dterms):
         PolSolve=[True],
## Data have been corrected for parallactic angle:
         parang_corrected=True,
         target_field = 'POLSIM',
## Apply the full non-linear Dterm model:
         linear_approx=False)

## Change the name of the Dterm CASA table, to keep it:
os.system('rm -rf PolSolve_PointSource.Dterms')
os.system('cp -r %s.spw_0.Dterms PolSolve_PointSource.Dterms'%MSNAME)



## READ FITTED DTERMS:
tb.open('%s.spw_0.Dterms'%MSNAME)
DTS = tb.getcol('CPARAM')
tb.close()



## RECOVER SIMULATED DTERMS:

INFILE = open('POLSIM_CASE-33_Dterms.dat','rb')
DRs,DLs = pk.load(INFILE) ; INFILE.close()




#### PLOT TRUE vs. FITTED DTERM COMPARISON:
fig = pl.figure()
sub = fig.add_subplot(111)
for ant in range(8):
  sub.plot(DRs.real,DTS[0,0,:].real,'or')
  sub.plot(DRs.imag,DTS[0,0,:].imag,'sr')
  sub.plot(DLs.real,DTS[1,0,:].real,'ob')
  sub.plot(DLs.imag,DTS[1,0,:].imag,'sb')

sub.plot([],[],'or',label='DR Real')
sub.plot([],[],'sr',label='DR Imag')
sub.plot([],[],'ob',label='DL Real')
sub.plot([],[],'sb',label='DL Imag')
sub.plot(np.array([-0.5,0.5]),np.array([-0.5,0.5]),':k')
pl.legend(numpoints=1,loc=4)
pl.xlabel('True Dterms')
pl.ylabel('PolSolve Dterms')
pl.suptitle('Point Source Calibrator')
pl.savefig('POLSOLVE_TEST.png')
pl.show()










